#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MODULE 0001
#define BOOST_TEST_NO_MAIN

#include <boost/test/unit_test.hpp>
#include <AMDiS.h>

using namespace AMDiS;
using namespace std;

BOOST_AUTO_TEST_CASE(blub)
{
  BOOST_REQUIRE(MPI::COMM_WORLD.Get_size() == 16);

  ProblemStat ellipt("ellipt");
  ellipt.initialize(INIT_ALL);
  MeshDistributor::globalMeshDistributor->initParallelization();

  Mesh* mesh = ellipt.getMesh();
  BOOST_REQUIRE(mesh->getNumberOfElements() == 2);
  BOOST_REQUIRE(mesh->getNumberOfLeaves() == 2);

  std::set<int> levelRanks = 
    MeshDistributor::globalMeshDistributor->getMeshLevelData().getLevelRanks(0);
  std::set<int> rankTest;
  rankTest.insert(-1);

  BOOST_REQUIRE(levelRanks == rankTest);

  std::set<int> levelNeighbours = 
    MeshDistributor::globalMeshDistributor->getMeshLevelData().getLevelNeighbours(0);
  std::set<int> neighTest;
  switch (MPI::COMM_WORLD.Get_rank()) {
  case 0:
    neighTest.insert(1); neighTest.insert(4); neighTest.insert(5);
    break;
  case 1:
    neighTest.insert(0); neighTest.insert(2); neighTest.insert(5); neighTest.insert(6);
    break;
  case 2:
    neighTest.insert(1); neighTest.insert(3); neighTest.insert(6); neighTest.insert(7);
    break;
  case 3:
    neighTest.insert(2); neighTest.insert(7);
    break;
  case 4:
    neighTest.insert(0); neighTest.insert(5); neighTest.insert(8); neighTest.insert(9);
    break;
  case 5:
    neighTest.insert(0); neighTest.insert(1); neighTest.insert(4); neighTest.insert(6);
    neighTest.insert(9); neighTest.insert(10);
    break;
  case 6:
    neighTest.insert(1); neighTest.insert(2); neighTest.insert(5); neighTest.insert(7);
    neighTest.insert(10); neighTest.insert(11);
    break;
  case 7:
    neighTest.insert(2); neighTest.insert(3); neighTest.insert(6); neighTest.insert(11);
    break;
  case 8:
    neighTest.insert(4); neighTest.insert(9); neighTest.insert(12); neighTest.insert(13);
    break;
  case 9:
    neighTest.insert(4); neighTest.insert(5); neighTest.insert(8); neighTest.insert(10);
    neighTest.insert(13); neighTest.insert(14);
    break;
  case 10:
    neighTest.insert(5); neighTest.insert(6); neighTest.insert(9); neighTest.insert(11);
    neighTest.insert(14); neighTest.insert(15);
    break;
  case 11:
    neighTest.insert(6); neighTest.insert(7); neighTest.insert(10); neighTest.insert(15);
    break;
  case 12:
    neighTest.insert(8); neighTest.insert(13);
    break;
  case 13:
    neighTest.insert(8); neighTest.insert(9); neighTest.insert(12); neighTest.insert(14);
    break;
  case 14:
    neighTest.insert(9); neighTest.insert(10); neighTest.insert(13); neighTest.insert(15);
    break;
  case 15:
    neighTest.insert(10); neighTest.insert(11); neighTest.insert(14);
    break;
  } 

  BOOST_REQUIRE(levelNeighbours == neighTest);
}


int main(int argc, char **argv)
{
  AMDiS::init(argc, argv, "./init/test0001.dat.2d");

  boost::unit_test::unit_test_main(&init_unit_test, argc, argv);

  AMDiS::finalize();
}
